// Copyright 2011-2012 Daniele Panozzo, panozzo@inf.ethz.ch. All rights reserved.
// The code in this file cannot be used for commercial
// applications without prior written permission from Daniele Panozzo

#ifndef VISUALIZER_H
#define VISUALIZER_H

#include <GL/glew.h>

/// app

#include <QtCore/QDebug>
#include <QtGui/QImage>

#include <Eigen/Core>
#include <string.h>

/// opengl
#ifdef __APPLE__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

typedef Eigen::Vector3d Point3d;
typedef Eigen::Vector2d Point2d;

using namespace std;

class Visualizer
{

public:
    Visualizer (int w, int h);

    // GLUT event handlers
    void initializeGL ();
    void resizeGL (int w, int h);
    void paintGL ();
    void mousePress (int button, int state, int x, int y);
    void mouseMove (int x, int y);
	void keyPress(unsigned char k);

    void renderToFile(const char* str);
    void saveSnapshot(const char* str);
    
    void paintCropBorder();
    void paintGL_aux(Eigen::VectorXd& x, Eigen::VectorXd& y, Eigen::VectorXd& u, Eigen::VectorXd& v, bool textureEnabled, bool saliencyEnabled, bool meshEnabled, bool disableMipmap = false, bool enableTextureCrop = false);

    // UI STATES
    bool isPainting;
    
    // Camera Control
    
	void setView(const Point2d& center, float radius); // center the view on the specified point
	void resetView();                           // show the entire image
	
    void updateGL ();
    void initMesh ();

	Point3d project(const Point3d& p);
	Point3d unproject(double sx, double sy, double sz);
	
    float gridColor[3];   // Color used for the grid

	// Left viewport
    bool meshEnabled;      // render the grid
    bool textureEnabled;   // render the image
    bool saliencyEnabled;  // render the saliency map

    // Right viewport
    bool meshEnabled2;      // render the grid
    bool textureEnabled2;   // render the image
    bool saliencyEnabled2;  // render the saliency map

    int width;  // window width
    int height; // window height

	void renderText(Point3d p, string str, bool _small); // render Text using glut
	string convertInt(int number);
	string convertDouble(double number);

	Point2d m_viewCenter;
	float m_viewRadius;
};

#endif // VISUALIZER_H
